<?php
abstract class FW_Form_Element_Abstract
{
  protected $validators = array();
  protected $decorators = array();
  protected $parameters = array();
  
  public function addValidator(FW_Validator_Abstract $validator)
  {
    $this->validators[] = $validator;    
    return $this;
  }

  final public function getHTML()
  {
    $html = "";
    foreach($this->decorators as $decorator)
    {
      $html .= $decorator->getHTMlBefore($this);
    }
    $html .= $this->getHTMLofElement();
    foreach($this->decorators as $decorator)
    {
      $html .= $decorator->getHTMlAfter($this);
    }
    return $html; 
  }
  
  abstract public function getHTMLofElement();
  
  public function validate()
  {
    return false;
  }
  
  public function setParameter($name, $value)
  {
    $this->parameters[$name] = $value;
    return $this;
  }
  
  public function setName($name)
  {
    $this->setParameter("name", $name);
    return $this;
  }
  
  public function getName()
  {
    return $this->getParameter("name");
  }
  
  public function getID()
  {
    return $this->getParameter("id");
  }
  
  public function setID($id)
  {
    $this->setParameter("id", $id);
    return $this;
  }
  
  public function setValue($value)
  {
    $this->setParameter("value", $value);
    return $this;
  }
  
  public function getValue()
  {
    return $this->getParameter("value");
  }
  
  public function getParameter($name)
  {
     return (isset($this->parameters[$name])) ? $this->parameters[$name] : null;
  }
  
  protected function getFurtherParameters()
  {
    $html = "";
    foreach($this->parameters as $name => $value)
    {
      $html .= " ".$name."='".$value."'";
    }
    return $html;
  }
  
  public function addDecorator(FW_Form_Decorator_Abstract $decorator, $amount = 1)
  {
    for($i = 0; $i < (int)$amount; $i++)
    {
      $this->decorators[] = $decorator;
    }  
    return $this;
  }
}
?>